"""Module providing functions for data postprocessing."""
# Import standard modules.

# Import own modules.
from datapostprocessingmodule import method_data_postprocessing
from datapostprocessingmodule import write_key_data_to_aircraft_exchange_file
from datapostprocessingmodule import prepare_element_tree_for_module_key_parameter

from .tube_and_wing.standard.basic.massPropertiesIO import MassPropertiesIO


def data_postprocessing(paths_and_names, routing_dict, data_dict, runtime_output):
    """Perform data postprocessing and write data to an aircraft exchange file.

    This function is responsible for data postprocessing that involves the following steps:
        (1) Data preparation: The module manager prepares a list containing all paths to the key parameters that must
        be written to the aircraft exchange file.
        (2) Write data to the aircraft exchange file: The results of the module execution are passed on to the function
        responsible for properly writing the data to the aircraft exchange file.
        (3) Method-specific data postprocessing: User-defined method-specific postprocessing is conducted as needed.

    :param dict paths_and_names: Dictionary containing system paths and ElementTrees
    :param dict routing_dict: Dictionary containing routing parameter
    :param dict data_dict: Dictionary containing the result of the module execution (direct operating cost estimation)
    :return: None
    """

    """Data preparation."""
    # Changes to this list are the sole responsibility of the module manager!
    paths_to_key_parameters_list = []
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("manufacturer_mass_empty"))
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("operating_mass_empty"))
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("maximum_takeoff_mass"))
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("maximum_zero_fuel_mass"))
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("maximum_payload_mass"))
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("maximum_landing_mass"))
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("maximum_fuel_mass"))
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("most_forward_mass"))
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("most_afterward_mass"))
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("design_mass"))
    paths_to_key_parameters_list.extend(
        MassPropertiesIO().key_parameters_paths("design_fuel_mass"))

    child_key_parameters_dict = {}
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("manufacturer_mass_empty", "MME"))
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("operating_mass_empty", "OME"))
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("maximum_takeoff_mass", "MTOM"))
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("maximum_zero_fuel_mass", "MZFM"))
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("maximum_payload_mass", ""))
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("maximum_landing_mass", "MLM"))
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("maximum_fuel_mass", ""))
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("most_forward_mass", ""))
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("most_afterward_mass", ""))
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("design_mass", ""))
    child_key_parameters_dict.update(
        MassPropertiesIO().key_parameters_dicts("design_fuel_mass", ""))

    module_key_parameters_dict = {
        "analysis": {
            "masses_cg_inertia": {
                "attributes": {"description": "Masses, Center of Gravity, Inertias.",
                               "tool_level": "0"},
                **child_key_parameters_dict}
        }
    }

    paths_and_names = prepare_element_tree_for_module_key_parameter(
        paths_and_names, module_key_parameters_dict)

    # Run 'user_method_data_output_preparation' from 'usermethoddatapreparation.py'.
    key_output_dict, method_specific_output_dict = routing_dict['func_user_method_data_output_preparation'](
        data_dict)
    # Extract tool level from routing dictionary.
    tool_level = routing_dict['tool_level']

    """Write data to aircraft exchange file."""
    # Extract root and path to aircraft exchange file.
    root_of_aircraft_exchange_tree = paths_and_names['root_of_aircraft_exchange_tree']
    path_to_aircraft_exchange_file = paths_and_names['path_to_aircraft_exchange_file']
    # Write key data to aircraft exchange file.
    write_key_data_to_aircraft_exchange_file(root_of_aircraft_exchange_tree, path_to_aircraft_exchange_file,
                                             paths_to_key_parameters_list, key_output_dict, tool_level, runtime_output)

    """Method-specific postprocessing."""
    # Run 'method_data_postprocessing' from 'datapostprocessingmodule'.
    method_data_postprocessing(paths_and_names, routing_dict, data_dict,
                               method_specific_output_dict, runtime_output)
