"""Module providing report functionalities for current calculation method."""
from yattag import Doc
from bs4 import BeautifulSoup
import re


def method_html_report(paths_and_names, routing_dict, data_dict, method_specific_output_dict, runtime_output):
    """Report function.

    This function is responsible for creating reports.
    [Add further information here...]

    :param dict paths_and_names: Dictionary containing system paths and ElementTrees
    :param dict routing_dict: Dictionary containing routing parameter
    :param dict data_dict: Dictionary containing results of module execution
    :return: None
    """

    # This is just a dummy code snippet. Insert your code here.
    tool_name = paths_and_names['tool_name']
    report_title = camelcase_to_string(tool_name)
    project_name = paths_and_names['name_of_project']

    runtime_output.print(f'Creating {tool_name} report')

    doc, tag, text = generate_report_header(
        report_title, project_name=project_name)

    # general aircraft masses
    aircraft_general_masses = [
        ("Manufacturer Mass Empty", "MME",
         data_dict["manufacturer_mass_empty"].mass),
        ("Operating Mass Empty", "OME",
         data_dict["operating_mass_empty"].mass),
        ("Maximum Takeoff Mass", "MTOM",
         data_dict["maximum_takeoff_mass"].mass),
        ("Maximum Zero Fuel Mass", "MZFM",
         data_dict["maximum_zero_fuel_mass"].mass),
        ("Maximum Payload Mass", "-", data_dict["maximum_payload_mass"].mass),
        ("Maximum Fuel mass", "-", data_dict["maximum_fuel_mass"].mass),
        ("Most forward mass", "-", data_dict["most_forward_mass"].mass),
        ("Most afterward mass", "-", data_dict["most_afterward_mass"].mass)
    ]

    aircraft_design_masses = [
        ("Maximum Takeoff Mass", "MTOM",
         data_dict["maximum_takeoff_mass"].mass),
        ("Maximum Landing Mass", "MLM",
         data_dict["maximum_landing_mass"].mass),
        ("Design Payload Mass", "-", data_dict["design_payload_mass"].mass),
        ("Design Fuel Mass Takeoff", "-",
         data_dict["design_fuel_mass_takeoff"].mass),
        ("Design Fuel Mass Midflight", "-",
         data_dict["design_fuel_mass_midflight"].mass),
        ("Design Fuel Mass Minimum Landing", "-",
         data_dict["design_fuel_mass_minimum_landing"].mass),
    ]

    aircraft_group_masses = []
    ome = data_dict["operating_mass_empty"].mass
    mtom = data_dict["maximum_takeoff_mass"].mass
    for key, mass in data_dict["group_masses"].items():
        name = key.replace("_", " ").title()
        aircraft_group_masses.append(
            (name, mass, mass / ome * 100.0, mass / mtom * 100.0))

    aircraft_structure_masses = []
    for key, mass in data_dict["structure_components"].items():
        name = key.replace("_", " ").title()
        aircraft_structure_masses.append(
            (name, mass, mass / ome * 100.0, mass / mtom * 100.0))

    aircraft_propulsion_masses = []
    nacelles = data_dict["Nacelles"]
    name = "Nacelles"
    mass = sum([nacelle.mass for nacelle in nacelles if nacelle is not None])
    aircraft_propulsion_masses.append(
        (name, mass, mass / ome * 100.0, mass / mtom * 100.0))

    engines = data_dict["Engines"]
    mass = sum([engine.mass for engine in engines if engine is not None])
    name = "Engines"
    aircraft_propulsion_masses.append(
        (name, mass, mass / ome * 100.0, mass / mtom * 100.0))

    def format_variable_name(var_name):
        # Replace underscores with spaces
        formatted_name = var_name.replace('_', ' ')
        formatted_name = formatted_name.replace('mass', ' ')
        # Capitalize the first letter of each word
        formatted_name = formatted_name.title()
        return formatted_name

    aircraft_systems_masses = []
    for item in data_dict["Systems"]:
        name = format_variable_name(item['name'])
        mass = item['mass']
        aircraft_systems_masses.append(
            (name, mass, mass / ome * 100.0, mass / mtom * 100.0))

    aircraft_furnishing_masses = []
    for item in data_dict["Furnishings"]:
        name = format_variable_name(item.name)
        aircraft_furnishing_masses.append(
            (name, item.mass, item.mass / ome * 100.0, item.mass / mtom * 100.0))

    aircraft_operator_masses = []
    for item in data_dict["Operator_items"]:
        name = format_variable_name(item.name)
        aircraft_operator_masses.append(
            (name, item.mass, item.mass / ome * 100.0, item.mass / mtom * 100.0))

    # Start generation
    with tag('body'):
        # First div - table aircraft general masses
        with tag('div', klass="logo"):
            text("")

        with tag('div', klass="content"):
            with tag('h1'):
                text(f"Report - {report_title} of {project_name}")
            with tag('div', klass="container"):
                with tag('div', klass="box data"):
                    with tag('h2'):
                        text("Data")

                    with tag('table', klass="content-table"):
                        with tag('caption'):
                            text("Aircraft General Masses")
                        with tag('thead'):
                            with tag('tr'):
                                for col in ["Description", "Acronym", "Mass [kg]"]:
                                    with tag('th'):
                                        text(f"{col}")
                        with tag('tbody'):
                            for row in aircraft_general_masses:
                                with tag('tr'):
                                    for col in row:
                                        if isinstance(col, (float, int)) or col is None:
                                            with tag('td', klass="content-data-number"):
                                                if col is None:
                                                    text('N.A.')
                                                else:
                                                    text(f"{col:.0f}")
                                        else:
                                            with tag('td'):
                                                if isinstance(col, str):
                                                    text(col)
                                                else:
                                                    text('N.A.')
                # Second div - table aircraft design masses

                    with tag('table', klass="content-table"):
                        with tag('caption'):
                            text("Aircraft Design Masses")
                        with tag('thead'):
                            with tag('tr'):
                                for col in ["Description", "Acronym", "Mass [kg]"]:
                                    with tag('th'):
                                        text(f"{col}")
                        with tag('tbody'):
                            for row in aircraft_design_masses:
                                with tag('tr'):
                                    for col in row:
                                        if isinstance(col, (float, int)) or col is None:
                                            with tag('td', klass="content-data-number"):
                                                if col is None:
                                                    text('N.A.')
                                                else:
                                                    text(f"{col:.0f}")
                                        else:
                                            with tag('td'):
                                                if isinstance(col, str):
                                                    text(col)
                                                else:
                                                    text('N.A.')

                    with tag('table', klass="content-table"):
                        with tag('caption'):
                            text("Operating Empty Mass Breakdown")
                        with tag('thead'):
                            with tag('tr'):
                                for col in ["Group", "Mass [kg]", "% of OME", "% of MTOM"]:
                                    with tag('th'):
                                        text(f"{col}")
                        with tag('tbody'):
                            for row in aircraft_group_masses:
                                with tag('tr'):
                                    for col in row:
                                        if isinstance(col, (float, int)) or col is None:
                                            with tag('td', klass="content-data-number"):
                                                if col is None:
                                                    text('N.A.')
                                                else:
                                                    text(f"{col:.1f}")
                                        else:
                                            with tag('td'):
                                                if isinstance(col, str):
                                                    text(col)

                    with tag('table', klass="content-table"):
                        with tag('caption'):
                            text(" Structure Group Mass Breakdown")
                        with tag('thead'):
                            with tag('tr'):
                                for col in ["Component", "Mass [kg]", "% of OME", "% of MTOM"]:
                                    with tag('th'):
                                        text(f"{col}")
                        with tag('tbody'):
                            for row in aircraft_structure_masses:
                                with tag('tr'):
                                    for col in row:
                                        if isinstance(col, (float, int)) or col is None:
                                            with tag('td', klass="content-data-number"):
                                                if col is None:
                                                    text('N.A.')
                                                else:
                                                    text(f"{col:.1f}")
                                        else:
                                            with tag('td'):
                                                if isinstance(col, str):
                                                    text(col)

                    with tag('table', klass="content-table"):
                        with tag('caption'):
                            text(" Propulsion Group Mass Breakdown")
                        with tag('thead'):
                            with tag('tr'):
                                for col in ["Component", "Mass [kg]", "% of OME", "% of MTOM"]:
                                    with tag('th'):
                                        text(f"{col}")
                        with tag('tbody'):
                            for row in aircraft_propulsion_masses:
                                with tag('tr'):
                                    for col in row:
                                        if isinstance(col, (float, int)) or col is None:
                                            with tag('td', klass="content-data-number"):
                                                if col is None:
                                                    text('N.A.')
                                                else:
                                                    text(f"{col:.1f}")
                                        else:
                                            with tag('td'):
                                                if isinstance(col, str):
                                                    text(col)

                    with tag('table', klass="content-table"):
                        with tag('caption'):
                            text(" Systems Mass Breakdown")
                        with tag('thead'):
                            with tag('tr'):
                                for col in ["Component", "Mass [kg]", "% of OME", "% of MTOM"]:
                                    with tag('th'):
                                        text(f"{col}")
                        with tag('tbody'):
                            for row in aircraft_systems_masses:
                                with tag('tr'):
                                    for col in row:
                                        if isinstance(col, (float, int)) or col is None:
                                            with tag('td', klass="content-data-number"):
                                                if col is None:
                                                    text('N.A.')
                                                else:
                                                    text(f"{col:.1f}")
                                        else:
                                            with tag('td'):
                                                if isinstance(col, str):
                                                    text(col)

                    with tag('table', klass="content-table"):
                        with tag('caption'):
                            text(" Furnishings Mass Breakdown")
                        with tag('thead'):
                            with tag('tr'):
                                for col in ["Component", "Mass [kg]", "% of OME", "% of MTOM"]:
                                    with tag('th'):
                                        text(f"{col}")
                        with tag('tbody'):
                            for row in aircraft_furnishing_masses:
                                with tag('tr'):
                                    for col in row:
                                        if isinstance(col, (float, int)) or col is None:
                                            with tag('td', klass="content-data-number"):
                                                if col is None:
                                                    text('N.A.')
                                                else:
                                                    text(f"{col:.1f}")
                                        else:
                                            with tag('td'):
                                                if isinstance(col, str):
                                                    text(col)

                    with tag('table', klass="content-table"):
                        with tag('caption'):
                            text(" Operator Items Mass Breakdown")
                        with tag('thead'):
                            with tag('tr'):
                                for col in ["Component", "Mass [kg]", "% of OME", "% of MTOM"]:
                                    with tag('th'):
                                        text(f"{col}")
                        with tag('tbody'):
                            for row in aircraft_operator_masses:
                                with tag('tr'):
                                    for col in row:
                                        if isinstance(col, (float, int)) or col is None:
                                            with tag('td', klass="content-data-number"):
                                                if col is None:
                                                    text('N.A.')
                                                else:
                                                    text(f"{col:.1f}")
                                        else:
                                            with tag('td'):
                                                if isinstance(col, str):
                                                    text(col)

                with tag('div', klass="box plot"):
                    with tag('h2'):
                        text("Plot")
                    with tag('img', src="../plots/aircraft_group_masses.svg", klass="image-plot"):
                        pass
                    with tag('img', src="../plots/weight_and_balance_diagramm.svg", klass="image-plot"):
                        pass
                    with tag('img', src="../plots/cg_cases.svg", klass="image-plot"):
                        pass

    soup = BeautifulSoup(doc.getvalue(), 'html.parser')
    html_output = soup.prettify()

    # Define the path where the report will be saved
    output_path = paths_and_names["project_directory"] + \
        '/reporting/reportHTML/' + f"{tool_name}_report.html"

    # Save the HTML content to the specified path
    with open(output_path, "w", encoding="utf-8") as report:
        report.write(html_output)


def generate_report_header(tool_name, project_name):

    doc, tag, text = Doc().tagtext()

    with tag('html', lang="en"):
        with tag('head'):
            doc.stag('meta', name="viewport",
                     content="width=device-width, initial-scale=1.0")
            doc.stag('meta', charset="UTF-8")
            with tag('title'):
                text(tool_name)
            doc.stag('link', rel="stylesheet", href="style.css")

    return doc, tag, text


def generate_report():
    pass


def camelcase_to_string(string):
    s_with_spaces = re.sub(r"(\w)([A-Z])", r"\1 \2", string)
    # Split, capitalize, and join
    return ' '.join(word.capitalize() for word in s_with_spaces.split())
