"""Module providing functions for the preparation of user data."""


def user_method_data_input_preparation(routing_dict):
    """Prepare necessary input data for the user method from aircraft exchange and module configuration files.

    In this function, the user is responsible for preparing the data needed for the user method. Relevant general data
    are obtained from the aircraft exchange file and calculation specific parameter from the module configuration file.
    The user must submit the data in the following format:
        dict = {'parameter_name': [path to parameter node, expected data type], ...}

    :param dict routing_dict: Dictionary containing information on necessary data from module configuration file
    :returns:
        - dict data_to_extract_from_aircraft_exchange_dict: Dictionary containing parameter name, path to parameter,
        and expected data type of parameters to be extracted from aircraft exchange file
        - dict data_to_extract_from_module_configuration_dict: Dictionary containing parameter name, path to parameter,
        and expected data type of parameters to be extracted from module configuration file
    """

    """Aircraft exchange file."""
    # Enter all parameters to be extracted from the aircraft exchange file.
    requirements_wing_path = \
        './requirements_and_specifications/design_specification/configuration/wing_definition/'
    component_design_path = './component_design/'
    mission_analysis_path = './analysis/mission/design_mission/'
    aero_analysis_path = './analysis/aerodynamics/reference_values/'
    top_level_aircraft_requirements_path = \
        './requirements_and_specifications/requirements/top_level_aircraft_requirements/'
    design_specification_path = \
        './requirements_and_specifications/design_specification/'
    data_to_extract_from_aircraft_exchange_dict = {
        "maximum_structural_payload_mass": [
            top_level_aircraft_requirements_path + 'maximum_structrual_payload_mass', float],
        "wing_exist": [component_design_path + "wing", None],
        "fuselage_exist": [component_design_path + "fuselage", None],
        "empennage_exist": [component_design_path + "empennage", None],
        "landing_gear_exist": [component_design_path + "landing_gear", None],
        "propulsion_exist": [component_design_path + "propulsion", None],
        "systems_exist": [component_design_path + "systems", None],
        "wing_mounting": [requirements_wing_path + 'mounting', str],
        "x_global_reference_point": [component_design_path + 'global_reference_point/x', float],
        "x_wing_globlal_position": [component_design_path + 'wing/position/x', float],
        "x_aircraft_neutral_point": [aero_analysis_path + 'neutral_point/x', float],
        "aircraft_type": [design_specification_path + "configuration/configuration_type", str],
        "loaded_mission_energy": [
            mission_analysis_path + 'loaded_mission_energy/mission_energy[@ID="0"]/consumed_energy', float],
        "loaded_mission_energy_carrier": [
            mission_analysis_path + 'loaded_mission_energy/mission_energy[@ID="0"]/energy_carrier_ID', float],
        "taxi_energy_in": [
            mission_analysis_path + 'taxi_energy/taxi_in_energy[@ID="0"]/consumed_energy', float],
        "taxi_energy_in_carrier": [
            mission_analysis_path + 'taxi_energy/taxi_in_energy[@ID="0"]/energy_carrier_ID', float],
        "taxi_energy_out": [
            mission_analysis_path + 'taxi_energy/taxi_out_energy[@ID="0"]/consumed_energy', float],
        "taxi_energy_out_carrier": [
            mission_analysis_path + 'taxi_energy/taxi_out_energy[@ID="0"]/energy_carrier_ID', float],
        "take_off_energy": [
            mission_analysis_path + 'in_flight_energy/takoff_energy[@ID="0"]/consumed_energy', float],
        "take_off_energy_carrier": [
            mission_analysis_path + 'in_flight_energy/takoff_energy[@ID="0"]/energy_carrier_ID', float],
        "landing_energy": [
            mission_analysis_path + 'in_flight_energy/landing_energy[@ID="0"]/consumed_energy', float],
        "landing_energy_carrier": [
            mission_analysis_path + 'in_flight_energy/landing_energy[@ID="0"]/energy_carrier_ID', float],
        "trip_energy": [
            mission_analysis_path + 'in_flight_energy/trip_energy[@ID="0"]/consumed_energy', float],
        "trip_energy_carrier": [
            mission_analysis_path + 'in_flight_energy/trip_energy[@ID="0"]/energy_carrier_ID', float],
        "energy_carrier_gravimetric_density": [design_specification_path +
                                               'energy_carriers/energy_carrier[@ID="0"]/gravimetric_density', float],
        "energy_carrier_volumetric_density": [design_specification_path +
                                              'energy_carriers/energy_carrier[@ID="0"]/volumetric_density', float],
        "energy_carrier_density": [design_specification_path + 'energy_carriers/energy_carrier[@ID="0"]/density',
                                   float],
        "energy_carrier_type": [design_specification_path + 'energy_carriers/energy_carrier[@ID="0"]/type', str],
    }

    """Module configuration file."""
    # Enter all general parameters to be extracted from the module configuration file. 'general parameters' means
    # parameters that do not differ according to the user layer. It should be noted that 'tmp_general' is only used to
    # shorten the path information in the 'general_data_to_extract_from_module_configuration_dict'.
    module_settings_path = f"./program_settings/tube_and_wing/standard/{routing_dict['user_layer']}/"
    general_data_to_extract_from_module_configuration_dict = {
        "mode_inertia_calculation": [module_settings_path + "calculation_methods/inertia/method", str],
        "aircraft_class": [module_settings_path + "calculation_methods/aircraft_type", str],
        "mode_maximum_landing_mass_calculation": [module_settings_path +
                                                  "calculation_methods/maximum_landing_mass/method", str],
        "refueling_mode": [module_settings_path + "calculation_methods/refueling_mode/method", str],
        "defueling_mode": [module_settings_path + "calculation_methods/defueling_mode/method", str],
        "passengers_boarding_mode": [module_settings_path + "calculation_methods/passengers_boarding_mode/method", str]
    }

    # Enter all specific parameters to be extracted from the module configuration file. 'specific parameters' means
    # parameters that differ according to the user layer. It should be noted that 'tmp_specific' is only used to
    # shorten the path information in the 'specific_data_to_extract_from_module_configuration_dict'.

    specific_data_to_extract_from_module_configuration_dict = {

    }

    # Merge module configuration dictionaries.
    data_to_extract_from_module_configuration_dict = \
        general_data_to_extract_from_module_configuration_dict | specific_data_to_extract_from_module_configuration_dict

    return data_to_extract_from_aircraft_exchange_dict, data_to_extract_from_module_configuration_dict


def user_method_data_output_preparation(data_dict):
    """Prepare user-specific output data based on the calculation method results.

    This function is responsible for preparing the user-specific output data based on the results of the calculation
    method. The 'data_dict' input parameter contains the results of the module execution.
    The data must be specified in the following format in order to be written correctly to the aircraft exchange file
    by the 'write_key_data_to_aircraft_exchange_file' function in the following step:
        dict = {'parameter_name': [path to parameter node, value, name (if needed)], ...}
    Important notes:
        (1) It should be noted that only key parameters may be written that have been previously defined by the module
        manager.
        (2) Attention must be paid to the proper path specifications, otherwise warnings may be issued or, in the worst
        case, errors may occur subsequently resulting in the write process and consequently the entire program being
        aborted.
        (3) If the path specifications contain IDs, these must start at '0' and be defined in ascending order without
        gaps.

    :param dict data_dict: Dictionary containing the results of the module execution
    :return dict user_output_dict: Dictionary containing parameter name, path to parameter, and value of key parameters
    """

    key_output_dict = {}
    key_output_dict.update(
        data_dict["manufacturer_mass_empty"].user_method_data_generation())
    key_output_dict.update(
        data_dict["operating_mass_empty"].user_method_data_generation())
    key_output_dict.update(
        data_dict["maximum_takeoff_mass"].user_method_data_generation())
    key_output_dict.update(
        data_dict["maximum_zero_fuel_mass"].user_method_data_generation())
    key_output_dict.update(
        data_dict["maximum_payload_mass"].user_method_data_generation())
    key_output_dict.update(
        data_dict["maximum_landing_mass"].user_method_data_generation())
    key_output_dict.update(
        data_dict["maximum_fuel_mass"].user_method_data_generation())
    key_output_dict.update(
        data_dict["most_forward_mass"].user_method_data_generation())
    key_output_dict.update(
        data_dict["most_afterward_mass"].user_method_data_generation())
    key_output_dict.update(
        data_dict["design_mass"].user_method_data_generation())
    key_output_dict.update(
        data_dict["design_fuel_mass"].user_method_data_generation())

    method_specific_output_dict = {
        # List of paths
        # 'list_of_paths':

    }

    return key_output_dict, method_specific_output_dict
